'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

exports.observer = observer;

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _observerUtil = require('@nx-js/observer-util');

var _batching = require('../batching');

var _batching2 = _interopRequireDefault(_batching);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function NullComponent() {
  return null;
} // ref: https://github.com/mobxjs/mobx-react-lite/blob/master/src/observer.ts
function observer(baseComponent) {
  var baseComponentName = baseComponent.displayName || baseComponent.name;
  // memo; we are not intested in deep updates
  // in props; we assume that if deep objects are changed,
  // this is in observables, which would have been tracked anyway

  var memoComponent = React.memo(function (props) {
    // forceUpdate 2.0
    var forceUpdate = useForceUpdate();

    // wrap the baseComponent into an observe decorator once.
    // This way it will track any observable changes and will trigger rerender
    var observedComponent = React.useMemo(function () {
      var update = function update() {
        // TODO: Decide whether the check for unmount is needed here
        forceUpdate();
      };
      var batchedUpdate = function batchedUpdate() {
        return _batching2.default.add(update);
      };
      return (0, _observerUtil.observe)(baseComponent, {
        scheduler: batchedUpdate,
        lazy: true
      });
    }, []);

    // clean up observer on unmount
    useUnmount(function () {
      return (0, _observerUtil.unobserve)(observedComponent);
    });

    return observedComponent(props);
  });
  memoComponent.displayName = baseComponentName;
  if (baseComponent.propTypes) {
    memoComponent.propTypes = baseComponent.propTypes;
  }
  return function (props) {
    return React.createElement(React.Suspense, { fallback: React.createElement(NullComponent, null, null) }, React.createElement(memoComponent, props));
  };
  // return memoComponent
}

function useForceUpdate() {
  var _React$useState = React.useState(),
      _React$useState2 = (0, _slicedToArray3.default)(_React$useState, 2),
      setTick = _React$useState2[1];

  return function () {
    setTick(Math.random());
  };
}

// TODO: Might change to just `useEffect` in future. Don't know which one fits here better yet.
function useUnmount(fn) {
  React.useLayoutEffect(function () {
    return fn;
  }, []);
}